package me.lensfrex.littlebusters.api.v1;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.MediaType;
import me.lensfrex.littlebusters.api.v1.dao.UserInformation;
import me.lensfrex.littlebusters.api.v1.dao.UserRegister;
import me.lensfrex.littlebusters.api.v1.responses.RegisterResponseData;
import me.lensfrex.littlebusters.api.v1.responses.general.ErrorResponse;
import me.lensfrex.littlebusters.api.v1.responses.general.ResponseBase;
import me.lensfrex.littlebusters.api.v1.utils.InputChecker;
import me.lensfrex.littlebusters.api.v1.utils.MyBatisUtil;
import org.apache.ibatis.session.SqlSession;
import org.mindrot.jbcrypt.BCrypt;

import java.util.UUID;

@Path("/register")
public class Register {
    private final static Gson gson = new Gson();

    @POST
    @Produces(MediaType.APPLICATION_JSON)
    public String register(String request) {
        System.out.println("Start register.");
        RegisterRequestBody registerRequestBody;
        try {
            registerRequestBody = gson.fromJson(request, RegisterRequestBody.class);
            if (InputChecker.hasInvalidChar(registerRequestBody.userName) ||
                    InputChecker.hasInvisibleChar(registerRequestBody.password)) {
                ErrorResponse errorResponse = new ErrorResponse(1, "用户名或密码非法");

                return gson.toJson(errorResponse);
            }
        } catch (Exception e) {
            System.err.println(request);
            ErrorResponse errorResponse = new ErrorResponse(2, "请求的数据格式不对");

            return gson.toJson(errorResponse);
        }

        String userUUID = UUID.randomUUID().toString();
        String userBcryptPasswd = BCrypt.hashpw(registerRequestBody.password, BCrypt.gensalt());

        SqlSession sqlSession = MyBatisUtil.getSqlSession(true);
        UserRegister userRegister = sqlSession.getMapper(UserRegister.class);

        userRegister.addRegisterInfoIntoDb(userUUID, registerRequestBody.userName, userBcryptPasswd);

        RegisterResponseData registerResponseBody = new RegisterResponseData(10101, userUUID, userBcryptPasswd);
        ResponseBase<RegisterResponseData> response = new ResponseBase<>(200, "success", registerResponseBody);

        return gson.toJson(response);
    }

    public static class RegisterRequestBody {
        @SerializedName("user_name")
        private String userName;

        private String password;

        public RegisterRequestBody(String userName, String password) {
            this.userName = userName;
            this.password = password;
        }

        public String getUserName() {
            return userName;
        }

        public void setUserName(String userName) {
            this.userName = userName;
        }

        public String getPassword() {
            return password;
        }

        public void setPassword(String password) {
            this.password = password;
        }
    }
}